<?php
namespace nur\sery\schema;

use ArrayAccess;
use IteratorAggregate;
use nur\sery\schema\types\IType;

abstract class Value implements ArrayAccess, IteratorAggregate {
  /** spécifier la valeur destination gérée par cet objet */
  abstract function reset(&$dest, $destKey=null, bool $verifix=true): self;

  /**
   * Obtenir la liste des clés valides pour les valeurs accessibles via cet
   * objet
   */
  abstract function getKeys(): array;

  /** obtenir un objet pour gérer la valeur spécifiée */
  abstract function getValue($key=null): Value;

  function getIterator() {
    foreach ($this->getKeys() as $key) {
      yield $key => $this->getValue($key);
    }
  }

  /** retourner le type associé à la valeur */
  abstract function getType(): IType;

  /** retourner true si la valeur existe */
  abstract function exists(): bool;

  /** supprimer la valeur */
  abstract function unset(): void;

  /** remplacer la valeur */
  abstract function set($value): self;

  /** obtenir le résultat de l'appel de la fonction {@link set()} */
  abstract function result(): Result;

  /** retourner true si la valeur est valide */
  abstract function valid(): bool;

  /** obtenir la valeur */
  abstract function get($default=null);

  /** retourner true si la valeur est dans sa forme normalisée */
  abstract function normalized(): bool;

  /** formatter la valeur pour affichage */
  abstract function format($format=null): string;
}
